@@ -3,7 +3,7 @@
 # Univention Nagios
 #  listener module: update configuration of local Nagios client
 #
-# Copyright 2004-2021 Univention GmbH
+# Copyright 2004-2022 Univention GmbH
 #
 # https://www.univention.de/
 #
@@ -32,11 +32,13 @@
 
 from __future__ import absolute_import
 
-import listener
 import os
 import re
 import stat
-import univention.debug
+
+import univention.debug as ud
+
+from listener import SetUID, configRegistry, run
 
 name = 'nagios-client'
 description = 'Create configuration for Nagios nrpe server'
@@ -58,30 +60,27 @@ def readPluginConfig():
 		# save modification time
 		__pluginconfdirstat = os.stat(__pluginconfdir)[8]
 
-		univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'NAGIOS-CLIENT: updating plugin config')
+		ud.debug(ud.LISTENER, ud.INFO, 'NAGIOS-CLIENT: updating plugin config')
 
-		listener.setuid(0)
-		try:
+		with SetUID(0):
 			for fn in os.listdir(__pluginconfdir):
 				with open(os.path.join(__pluginconfdir, fn), 'rb') as fp:
-					content = fp.read()
-				for cmddef in re.split(r'\s*define\s+command\s*\{'.encode('ASCII'), content):
-					mcmdname = re.search(r'^\s+command_name\s+(.*?)\s*$'.encode('ASCII'), cmddef, re.MULTILINE)
-					mcmdline = re.search(r'^\s+command_line\s+(.*?)\s*$'.encode('ASCII'), cmddef, re.MULTILINE)
+					content = fp.read().decode('UTF-8', 'replace')
+				for cmddef in re.split(r'\s*define\s+command\s*\{', content):
+					mcmdname = re.search(r'^\s+command_name\s+(.*?)\s*$', cmddef, re.MULTILINE)
+					mcmdline = re.search(r'^\s+command_line\s+(.*?)\s*$', cmddef, re.MULTILINE)
 					if mcmdname and mcmdline:
 						__pluginconfig[mcmdname.group(1)] = mcmdline.group(1)
-						univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'NAGIOS-CLIENT: read configline for plugin %s ==> %s' % (mcmdname.group(1), mcmdline.group(1)))
-		finally:
-			listener.unsetuid()
+						ud.debug(ud.LISTENER, ud.INFO, 'NAGIOS-CLIENT: read configline for plugin %r ==> %r' % (mcmdname.group(1), mcmdline.group(1)))
 
 
 def replaceArguments(cmdline, args):
 	# type: (str, list) -> str
 	for i in range(9):
 		if i < len(args):
-			cmdline = re.sub(r'\$ARG%d\$'.encode('ASCII') % (i + 1), args[i], cmdline)
+			cmdline = re.sub(r'\$ARG%d\$' % (i + 1), args[i], cmdline)
 		else:
-			cmdline = re.sub(r'\$ARG%d\$'.encode('ASCII') % (i + 1), b'', cmdline)
+			cmdline = re.sub(r'\$ARG%d\$' % (i + 1), b'', cmdline)
 	return cmdline
 
 
@@ -90,27 +89,21 @@ def writeConfig(fqdn, new):
 	readPluginConfig()
 
 	name = new['cn'][0].decode('UTF-8')
-	cmdline = b'PluginNameNotFoundError'
+	cmdline = 'PluginNameNotFoundError'
 
 	# if no univentionNagiosHostname is present or current host is no member then quit
-	if 'univentionNagiosHostname' in new and new['univentionNagiosHostname']:
-		if fqdn not in new['univentionNagiosHostname']:
-			return
-	else:
+	if fqdn.encode('UTF-8') not in new.get('univentionNagiosHostname', []):
 		return
 
-	if 'univentionNagiosCheckCommand' in new and new['univentionNagiosCheckCommand'] and new['univentionNagiosCheckCommand'][0]:
-		if new['univentionNagiosCheckCommand'][0] in __pluginconfig:
-			cmdline = __pluginconfig[new['univentionNagiosCheckCommand'][0]]
-	if 'univentionNagiosCheckArgs' in new and new['univentionNagiosCheckArgs'] and new['univentionNagiosCheckArgs'][0]:
-		cmdline = replaceArguments(cmdline, new['univentionNagiosCheckArgs'][0].split(b'!'))
-	cmdline = re.sub(r'\$HOSTADDRESS\$'.encode('ASCII'), fqdn, cmdline)
-	cmdline = re.sub(r'\$HOSTNAME\$'.encode('ASCII'), fqdn, cmdline)
+	nagios_check_command = new.get('univentionNagiosCheckCommand', [b''])[0].decode('UTF-8')
+	cmdline = __pluginconfig.get(nagios_check_command, cmdline)
+	if new.get('univentionNagiosCheckArgs', [b''])[0]:
+		cmdline = replaceArguments(cmdline, new['univentionNagiosCheckArgs'][0].decode('UTF-8').split('!'))
+	cmdline = re.sub(r'\$HOSTADDRESS\$', fqdn, cmdline)
+	cmdline = re.sub(r'\$HOSTNAME\$', fqdn, cmdline)
 
-	listener.setuid(0)
-	try:
-		filename = os.path.join(__confdir, "%s.cfg" % name)
-		fp = open(filename, 'w')
+	filename = os.path.join(__confdir, "%s.cfg" % name)
+	with SetUID(0), open(filename, 'w') as fp:
 		fp.write('# Warning: This file is auto-generated and might be overwritten.\n')
 		fp.write('#          Please use univention-directory-manager instead.\n')
 		fp.write('# Warnung: Diese Datei wurde automatisch generiert und wird\n')
@@ -118,35 +111,28 @@ def writeConfig(fqdn, new):
 		fp.write('#          stattdessen den Univention Directory Manager.\n')
 		fp.write('\n')
 		fp.write('command[%s]=%s\n' % (name, cmdline))
-		fp.close()
-
-		univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'NAGIOS-CLIENT: service %s written' % name)
-	finally:
-		listener.unsetuid()
+	ud.debug(ud.LISTENER, ud.INFO, 'NAGIOS-CLIENT: service %s written' % name)
 
 
 def removeConfig(name):
 	# type: (str) -> None
 	filename = os.path.join(__confdir, "%s.cfg" % name)
-	listener.setuid(0)
-	try:
+	with SetUID(0):
 		if os.path.exists(filename):
 			os.unlink(filename)
-	finally:
-		listener.unsetuid()
 
 
 def handler(dn, new, old):
 	# type: (str, dict, dict) -> None
-	# univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'NAGIOS-CLIENT: IN dn=%r' % (dn,))
-	# univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'NAGIOS-CLIENT: IN old=%r' % (old,))
-	# univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'NAGIOS-CLIENT: IN new=%r' % (new,))
+	# ud.debug(ud.LISTENER, ud.INFO, 'NAGIOS-CLIENT: IN dn=%r' % (dn,))
+	# ud.debug(ud.LISTENER, ud.INFO, 'NAGIOS-CLIENT: IN old=%r' % (old,))
+	# ud.debug(ud.LISTENER, ud.INFO, 'NAGIOS-CLIENT: IN new=%r' % (new,))
 
-	fqdn = '%s.%s' % (listener.configRegistry['hostname'], listener.configRegistry['domainname'])
+	fqdn = '%(hostname)s.%(domainname)s' % configRegistry
 	fqdn = fqdn.encode('UTF-8')
 
 	if old and not new:
-		univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'NAGIOS-CLIENT: service %r deleted' % (old['cn'][0],))
+		ud.debug(ud.LISTENER, ud.INFO, 'NAGIOS-CLIENT: service %r deleted' % (old['cn'][0],))
 		removeConfig(old['cn'][0].decode('UTF-8'))
 
 	if old and \
@@ -156,7 +142,7 @@ def handler(dn, new, old):
 		# local fqdn was in old object and
 		# local fqdn is not in new object
 		# ==> fqdn was deleted from list
-		univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'NAGIOS-CLIENT: host removed from service %s' % (old['cn'][0],))
+		ud.debug(ud.LISTENER, ud.INFO, 'NAGIOS-CLIENT: host removed from service %s' % (old['cn'][0],))
 		removeConfig(old['cn'][0].decode('UTF-8'))
 	elif old and \
 		old.get('univentionNagiosUseNRPE', [None])[0] == b'1' and \
@@ -166,7 +152,7 @@ def handler(dn, new, old):
 		# NRPE was enabled in old object
 		# NRPE is disabled in new object
 		# ==> remove config
-		univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'NAGIOS-CLIENT: nrpe disabled for service %r' % (old['cn'][0],))
+		ud.debug(ud.LISTENER, ud.INFO, 'NAGIOS-CLIENT: nrpe disabled for service %r' % (old['cn'][0],))
 		removeConfig(old['cn'][0].decode('UTF-8'))
 	else:
 		# otherwise:
@@ -174,20 +160,17 @@ def handler(dn, new, old):
 		# - this host is newly added to list or
 		# - this object is new
 		if 'univentionNagiosUseNRPE' in new and new['univentionNagiosUseNRPE'] and (new['univentionNagiosUseNRPE'][0] == b'1'):
-			univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'NAGIOS-CLIENT: writing service %r' % (new['cn'][0],))
-			writeConfig(fqdn, new)
+			ud.debug(ud.LISTENER, ud.INFO, 'NAGIOS-CLIENT: writing service %r' % (new['cn'][0],))
+			writeConfig(fqdn.decode('UTF-8'), new)
 
 
+@SetUID(0)
 def initialize():
 	# type: () -> None
 	dirname = '/etc/nagios/nrpe.univention.d'
 
 	if not os.path.exists(dirname):
-		listener.setuid(0)
-		try:
-			os.mkdir(dirname)
-		finally:
-			listener.unsetuid()
+		os.mkdir(dirname)
 
 
 def deleteTree(dirname):
@@ -203,25 +186,18 @@ def deleteTree(dirname):
 		os.rmdir(dirname)
 
 
+@SetUID(0)
 def clean():
 	# type: () -> None
 	dirname = '/etc/nagios/nrpe.univention.d'
 	if os.path.exists(dirname):
-		listener.setuid(0)
-		try:
-			deleteTree(dirname)
-		finally:
-			listener.unsetuid()
+		deleteTree(dirname)
 
 
 def postrun():
 	# type: () -> None
 	global __initscript
 	initscript = __initscript
-	if listener.configRegistry.is_true("nagios/client/autostart"):
-		univention.debug.debug(univention.debug.LISTENER, univention.debug.INFO, 'NRPED: Restarting server')
-		listener.setuid(0)
-		try:
-			listener.run(initscript, ['nagios-nrpe-server', 'restart'], uid=0)
-		finally:
-			listener.unsetuid()
+	if configRegistry.is_true("nagios/client/autostart"):
+		ud.debug(ud.LISTENER, ud.INFO, 'NRPED: Restarting server')
+		run(initscript, ['nagios-nrpe-server', 'restart'], uid=0)
